@@ -8,10 +8,10 @@
 #include "logging.h"
 #include "notificationhelper.h"
 #include "pkpassmanager.h"
-#include "reservationhelper.h"
-#include "reservationmanager.h"
 #include "publictransport.h"
 #include "publictransportmatcher.h"
+#include "reservationhelper.h"
+#include "reservationmanager.h"
 
 #include <KItinerary/BusTrip>
 #include <KItinerary/Flight>
@@ -182,7 +182,7 @@ void LiveDataManager::checkForUpdates()
     return loc.hasCoordinate() || loc.country().size() == 2;
 }
 
-void LiveDataManager::checkReservation(const QVariant &res, const QString& resId)
+void LiveDataManager::checkReservation(const QVariant &res, const QString &resId)
 {
     using namespace KPublicTransport;
     const auto arrived = hasArrived(resId, res);
@@ -203,7 +203,9 @@ void LiveDataManager::checkReservation(const QVariant &res, const QString& resId
         PublicTransport::selectBackends(req, m_ptMgr, res);
         if (canSelectBackend(from) || canSelectBackend(to) || !req.backendIds().isEmpty()) {
             auto reply = m_ptMgr->queryJourney(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() { journeyQueryFinished(reply, resId); });
+            connect(reply, &Reply::finished, this, [this, resId, reply]() {
+                journeyQueryFinished(reply, resId);
+            });
             m_lastPollAttempt.insert(resId, now());
             return;
         }
@@ -216,7 +218,9 @@ void LiveDataManager::checkReservation(const QVariant &res, const QString& resId
         PublicTransport::selectBackends(req, m_ptMgr, res);
         if (canSelectBackend(req.stop()) || !req.backendIds().isEmpty()) {
             auto reply = m_ptMgr->queryStopover(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() { stopoverQueryFinished(reply, LiveData::Departure, resId); });
+            connect(reply, &Reply::finished, this, [this, resId, reply]() {
+                stopoverQueryFinished(reply, LiveData::Departure, resId);
+            });
             m_lastPollAttempt.insert(resId, now());
         }
     }
@@ -228,13 +232,15 @@ void LiveDataManager::checkReservation(const QVariant &res, const QString& resId
         PublicTransport::selectBackends(req, m_ptMgr, res);
         if (canSelectBackend(req.stop()) || !req.backendIds().isEmpty()) {
             auto reply = m_ptMgr->queryStopover(req);
-            connect(reply, &Reply::finished, this, [this, resId, reply]() { stopoverQueryFinished(reply, LiveData::Arrival, resId); });
+            connect(reply, &Reply::finished, this, [this, resId, reply]() {
+                stopoverQueryFinished(reply, LiveData::Arrival, resId);
+            });
             m_lastPollAttempt.insert(resId, now());
         }
     }
 }
 
-void LiveDataManager::stopoverQueryFinished(KPublicTransport::StopoverReply* reply, LiveData::Type type, const QString& resId)
+void LiveDataManager::stopoverQueryFinished(KPublicTransport::StopoverReply *reply, LiveData::Type type, const QString &resId)
 {
     reply->deleteLater();
     if (reply->error() != KPublicTransport::Reply::NoError) {
@@ -244,12 +250,13 @@ void LiveDataManager::stopoverQueryFinished(KPublicTransport::StopoverReply* rep
     stopoverQueryFinished(reply->takeResult(), type, resId);
 }
 
-void LiveDataManager::stopoverQueryFinished(std::vector<KPublicTransport::Stopover> &&result, LiveData::Type type, const QString& resId)
+void LiveDataManager::stopoverQueryFinished(std::vector<KPublicTransport::Stopover> &&result, LiveData::Type type, const QString &resId)
 {
     const auto res = m_resMgr->reservation(resId);
     for (const auto &stop : result) {
         qCDebug(Log) << "Got stopover information:" << stop.route().line().name() << stop.scheduledDepartureTime();
-        if (type == LiveData::Arrival ? PublicTransportMatcher::isArrivalForReservation(res, stop) : PublicTransportMatcher::isDepartureForReservation(res, stop)) {
+        if (type == LiveData::Arrival ? PublicTransportMatcher::isArrivalForReservation(res, stop)
+                                      : PublicTransportMatcher::isDepartureForReservation(res, stop)) {
             qCDebug(Log) << "Found stopover information:" << stop.route().line().name() << stop.expectedPlatform() << stop.expectedDepartureTime();
             updateStopoverData(stop, type, resId, res);
             return;
@@ -271,10 +278,17 @@ void LiveDataManager::journeyQueryFinished(KPublicTransport::JourneyReply *reply
     using namespace KPublicTransport;
     const auto res = m_resMgr->reservation(resId);
     for (const auto &journey : reply->result()) {
-        if (std::count_if(journey.sections().begin(), journey.sections().end(), [](const auto &sec) { return sec.mode() == JourneySection::PublicTransport; }) != 1) {
+        if (std::count_if(journey.sections().begin(),
+                          journey.sections().end(),
+                          [](const auto &sec) {
+                              return sec.mode() == JourneySection::PublicTransport;
+                          })
+            != 1) {
             continue;
         }
-        const auto it = std::find_if(journey.sections().begin(), journey.sections().end(), [](const auto &sec) { return sec.mode() == JourneySection::PublicTransport; });
+        const auto it = std::find_if(journey.sections().begin(), journey.sections().end(), [](const auto &sec) {
+            return sec.mode() == JourneySection::PublicTransport;
+        });
         assert(it != journey.sections().end());
         qCDebug(Log) << "Got journey information:" << (*it).route().line().name() << (*it).scheduledDepartureTime();
         if (PublicTransportMatcher::isJourneyForReservation(res, (*it))) {
@@ -367,7 +381,7 @@ static void applyMissingJourneyData(KPublicTransport::JourneySection &journey, c
     journey.setIntermediateStops(std::move(stops));
 
     if (!KPublicTransport::Stopover::isSame(journey.departure(), oldJny.departure())
-     || !KPublicTransport::Stopover::isSame(journey.arrival(), oldJny.arrival())) {
+        || !KPublicTransport::Stopover::isSame(journey.arrival(), oldJny.arrival())) {
         return;
     }
     auto s = journey.departure();
@@ -424,8 +438,8 @@ void LiveDataManager::updateJourneyData(const KPublicTransport::JourneySection &
     Q_EMIT arrivalUpdated(resId);
 
     // check if we need to notify
-    if (NotificationHelper::shouldNotify(oldDep, ld.journey.departure(), LiveData::Departure) ||
-        NotificationHelper::shouldNotify(oldArr, ld.journey.arrival(), LiveData::Arrival)) {
+    if (NotificationHelper::shouldNotify(oldDep, ld.journey.departure(), LiveData::Departure)
+        || NotificationHelper::shouldNotify(oldArr, ld.journey.arrival(), LiveData::Arrival)) {
         showNotification(resId, ld);
     }
 }
@@ -444,7 +458,7 @@ void LiveDataManager::showNotification(const QString &resId, const LiveData &ld)
     }
 }
 
-void LiveDataManager::fillNotification(KNotification* n, const LiveData& ld) const
+void LiveDataManager::fillNotification(KNotification *n, const LiveData &ld) const
 {
     n->setTitle(NotificationHelper::title(ld));
     n->setText(NotificationHelper::message(ld));
@@ -510,7 +524,7 @@ bool LiveDataManager::hasArrived(const QString &resId, const QVariant &res) cons
     return arrivalTime(resId, res) < now();
 }
 
-LiveData& LiveDataManager::data(const QString &resId) const
+LiveData &LiveDataManager::data(const QString &resId) const
 {
     auto it = m_data.find(resId);
     if (it != m_data.end()) {
@@ -521,7 +535,7 @@ LiveData& LiveDataManager::data(const QString &resId) const
     return it.value();
 }
 
-void LiveDataManager::importData(const QString& resId, LiveData &&data)
+void LiveDataManager::importData(const QString &resId, LiveData &&data)
 {
     // we don't need to store data, Importer already does that
     m_data[resId] = std::move(data);
@@ -678,7 +692,7 @@ int LiveDataManager::nextPollTime() const
     for (const auto &resId : m_reservations) {
         t = std::min(t, nextPollTimeForReservation(resId));
     }
-    qCDebug(Log) << "next auto-update in" << (t/1000) << "secs";
+    qCDebug(Log) << "next auto-update in" << (t / 1000) << "secs";
     return t;
 }
 
@@ -687,14 +701,14 @@ struct {
     int distance; // secs
     int pollInterval; // secs
 } static const pollIntervalTable[] = {
-    { 3600, 5*60 }, // for <1h we poll every 5 minutes
-    { 4 * 3600, 15 * 60 }, // for <4h we poll every 15 minutes
-    { 24 * 3600, 3600 }, // for <1d we poll once per hour
-    { 4 * 24 * 3600, 24 * 3600 }, // for <4d we poll once per day
-    { 60 * 24 * 3600, MAX_POLL_INTERVAL }, // anything before we should at least do one poll to get full details right away
+    {3600, 5 * 60}, // for <1h we poll every 5 minutes
+    {4 * 3600, 15 * 60}, // for <4h we poll every 15 minutes
+    {24 * 3600, 3600}, // for <1d we poll once per hour
+    {4 * 24 * 3600, 24 * 3600}, // for <4d we poll once per day
+    {60 * 24 * 3600, MAX_POLL_INTERVAL}, // anything before we should at least do one poll to get full details right away
 };
 
-int LiveDataManager::nextPollTimeForReservation(const QString& resId) const
+int LiveDataManager::nextPollTimeForReservation(const QString &resId) const
 {
     const auto res = m_resMgr->reservation(resId);
 
@@ -725,9 +739,8 @@ int LiveDataManager::nextPollTimeForReservation(const QString& resId) const
             lastRelevantPoll = lastDeparturePoll;
         }
     }
-    const int lastPollDist = (!lastRelevantPoll.isValid() || lastRelevantPoll > now)
-        ? MAX_POLL_INTERVAL // no poll yet == long time ago
-        : lastRelevantPoll.secsTo(now);
+    const int lastPollDist = (!lastRelevantPoll.isValid() || lastRelevantPoll > now) ? MAX_POLL_INTERVAL // no poll yet == long time ago
+                                                                                     : lastRelevantPoll.secsTo(now);
     return std::max((it->pollInterval - lastPollDist) * 1000, pollCooldown(resId)); // we need msecs
 }
 
@@ -796,7 +809,7 @@ void LiveDataManager::pkPassUpdated(const QString &passId, const QStringList &ch
     KNotification::event(KNotification::Notification, i18n("Itinerary change"), text, QLatin1StringView("clock"));
 }
 
-KPublicTransport::Manager* LiveDataManager::publicTransportManager() const
+KPublicTransport::Manager *LiveDataManager::publicTransportManager() const
 {
     return m_ptMgr;
 }
