@@ -20,9 +20,8 @@
 #include <KPublicTransport/Location>
 #include <KPublicTransport/Manager>
 
-#include <QGeoCoordinate>
-
-namespace KPublicTransport {
+namespace KPublicTransport
+{
 class Journey;
 class JourneySection;
 class RentalVehicle;
@@ -44,7 +43,7 @@ public:
     Q_INVOKABLE static KPublicTransport::Location locationFromPlace(const QVariant &place, const QVariant &reservation);
 
     /** Create a KItinerary place type from the given KPublicTransport::Location. */
-    template <typename T>
+    template<typename T>
     static T placeFromLocation(const KPublicTransport::Location &loc);
 
     /** Same as the above, but usable from QML. */
@@ -55,7 +54,7 @@ public:
      *  Unlike mergeStation(), this does not assume both sides refer to the same location, and @p location has precedence.
      *  Data from @p station is only considered if both sides refer to the same location.
      */
-    template <typename T>
+    template<typename T>
     static T updateToLocation(const T &place, const KPublicTransport::Location &loc);
 
     /** Creates a reservation from the given journey section. */
@@ -76,8 +75,7 @@ public:
     static QVariant mergeArrival(const QVariant &res, const KPublicTransport::Stopover &arr);
     static QVariant mergeJourney(const QVariant &res, const KPublicTransport::JourneySection &journey);
 
-    /** Checkes whether @p mode is a train or bus mode respectively. */
-    static bool isTrainMode(KPublicTransport::Line::Mode mode);
+    /** Checks whether @p mode is a bus or ship mode respectively. */
     static bool isBusMode(KPublicTransport::Line::Mode mode);
     static bool isBoatMode(KPublicTransport::Line::Mode mode);
 
@@ -99,7 +97,7 @@ public:
     static KPublicTransport::JourneySection lastTransportSection(const KPublicTransport::Journey &journey);
 
     /** Selects suitable backends based on UIC operator codes from the reservation. */
-    template <typename Request>
+    template<typename Request>
     static void selectBackends(Request &request, KPublicTransport::Manager *mgr, const QVariant &res);
 
     /** Convert KPublicTransport mode enum to one for KOSMIndoorMap */
@@ -114,11 +112,6 @@ public:
      */
     [[nodiscard]] static bool isSameStopoverForLayout(const KPublicTransport::Stopover &lhs, const KPublicTransport::Stopover &rhs);
 
-    /** Convert a KPublicTransport::JourneySection path into a QGeoCoordinate list for consumption by a QtLocation
-     *  map item.
-     */
-    Q_INVOKABLE [[nodiscard]] static QList<QGeoCoordinate> pathToGeoCoordinates(const KPublicTransport::JourneySection &jny);
-
     /** Returns the maximum occupancy found in the given load information set. */
     Q_INVOKABLE [[nodiscard]] static KPublicTransport::Load::Category maximumOccupancy(const QList<KPublicTransport::LoadInfo> &loadInfo);
 
@@ -129,21 +122,20 @@ private:
     static QStringList suitableBackendsForReservation(KPublicTransport::Manager *mgr, const QVariant &res);
 };
 
-
-template <typename T>
+template<typename T>
 inline T PublicTransport::placeFromLocation(const KPublicTransport::Location &loc)
 {
     T place;
     place.setName(loc.name());
     if (loc.hasCoordinate()) {
-        place.setGeo(KItinerary::GeoCoordinates{loc.latitude(), loc.longitude()});
+        place.setGeo(KItinerary::GeoCoordinates{(float)loc.latitude(), (float)loc.longitude()});
     }
     place.setIdentifier(idenfitierFromLocation(loc));
     place.setAddress(addressFromLocation(loc));
     return place;
 }
 
-template <typename T>
+template<typename T>
 inline T PublicTransport::updateToLocation(const T &place, const KPublicTransport::Location &loc)
 {
     using namespace KItinerary;
@@ -156,7 +148,7 @@ inline T PublicTransport::updateToLocation(const T &place, const KPublicTranspor
     return newPlace;
 }
 
-template <typename Request>
+template<typename Request>
 inline void PublicTransport::selectBackends(Request &request, KPublicTransport::Manager *mgr, const QVariant &res)
 {
     request.setBackendIds(suitableBackendsForReservation(mgr, res));
