@@ -5,15 +5,12 @@
 */
 
 #include "timelinemodel.h"
-#include "constants.h"
 #include "locationhelper.h"
 #include "locationinformation.h"
-#include "pkpassmanager.h"
 #include "reservationmanager.h"
-#include "timelineelement.h"
+#include "transfermanager.h"
 #include "tripgroup.h"
 #include "tripgroupmanager.h"
-#include "transfermanager.h"
 #include "weatherinformation.h"
 
 #include "weatherforecast.h"
@@ -58,8 +55,7 @@ static bool needsSplitting(const QVariant &res)
         return true;
     }
 
-    return JsonLd::isA<LodgingReservation>(res)
-        || JsonLd::isA<RentalCarReservation>(res);
+    return JsonLd::isA<LodgingReservation>(res) || JsonLd::isA<RentalCarReservation>(res);
 }
 
 static QTimeZone timeZone(const QDateTime &dt)
@@ -78,7 +74,11 @@ TimelineModel::TimelineModel(QObject *parent)
 
     // make sure we properly update the empty today marker
     connect(this, &TimelineModel::todayRowChanged, this, [this]() {
-        const auto idx = index(todayRow(), 0);
+        const auto row = todayRow();
+        if (row < 0) {
+            return;
+        }
+        const auto idx = index(row, 0);
         if (m_todayEmpty == idx.data(TimelineModel::TodayEmptyRole).toBool()) {
             return;
         }
@@ -86,115 +86,113 @@ TimelineModel::TimelineModel(QObject *parent)
         Q_EMIT dataChanged(idx, idx);
     });
 
-    connect(&m_currentBatchTimer, &QTimer::timeout, this, &TimelineModel::currentBatchChanged);
-    connect(&m_currentBatchTimer, &QTimer::timeout, this, &TimelineModel::updateTodayMarker);
-    connect(&m_currentBatchTimer, &QTimer::timeout, this, &TimelineModel::scheduleCurrentBatchTimer);
-    m_currentBatchTimer.setTimerType(Qt::VeryCoarseTimer);
-    m_currentBatchTimer.setSingleShot(true);
+    connect(this, &TimelineModel::initialLocationChanged, this, &TimelineModel::updateWeatherElements);
 }
 
 TimelineModel::~TimelineModel() = default;
 
-void TimelineModel::setReservationManager(ReservationManager* mgr)
+void TimelineModel::setReservationManager(ReservationManager *mgr)
 {
-    // for auto tests only
-    if (Q_UNLIKELY(!mgr)) {
-        beginResetModel();
-        disconnect(m_resMgr, &ReservationManager::batchAdded, this, &TimelineModel::batchAdded);
-        disconnect(m_resMgr, &ReservationManager::batchChanged, this, &TimelineModel::batchChanged);
-        disconnect(m_resMgr, &ReservationManager::batchContentChanged, this, &TimelineModel::batchChanged);
-        disconnect(m_resMgr, &ReservationManager::batchRenamed, this, &TimelineModel::batchRenamed);
-        disconnect(m_resMgr, &ReservationManager::batchRemoved, this, &TimelineModel::batchRemoved);
-        m_resMgr = mgr;
-        m_elements.clear();
-        endResetModel();
+    if (m_resMgr == mgr) {
         return;
     }
+    assert(mgr);
 
-    beginResetModel();
     m_resMgr = mgr;
-    for (const auto &resId : mgr->batches()) {
-        const auto res = m_resMgr->reservation(resId);
-        auto elem = TimelineElement(this, resId, res, TimelineElement::SelfContained);
-        if (!elem.isReservation()) { // a type we can't handle
-            continue;
-        }
-        if (needsSplitting(res)) {
-            m_elements.push_back(TimelineElement{this, resId, res, TimelineElement::RangeBegin});
-            m_elements.push_back(TimelineElement{this, resId, res, TimelineElement::RangeEnd});
-        } else {
-            m_elements.push_back(std::move(elem));
-        }
-    }
-    m_elements.push_back(TimelineElement{this, TimelineElement::TodayMarker,  QDateTime(today(), QTime(0, 0))});
-    std::sort(m_elements.begin(), m_elements.end());
-
     connect(mgr, &ReservationManager::batchAdded, this, &TimelineModel::batchAdded);
     connect(mgr, &ReservationManager::batchChanged, this, &TimelineModel::batchChanged);
     connect(mgr, &ReservationManager::batchContentChanged, this, &TimelineModel::batchChanged);
     connect(mgr, &ReservationManager::batchRenamed, this, &TimelineModel::batchRenamed);
     connect(mgr, &ReservationManager::batchRemoved, this, &TimelineModel::batchRemoved);
-    endResetModel();
-
-    updateTodayMarker();
-    updateInformationElements();
-    Q_EMIT todayRowChanged();
-
-    scheduleCurrentBatchTimer();
-    Q_EMIT currentBatchChanged();
+    Q_EMIT setupChanged();
+    QMetaObject::invokeMethod(this, &TimelineModel::populate, Qt::QueuedConnection);
 }
 
-void TimelineModel::setWeatherForecastManager(WeatherForecastManager* mgr)
+void TimelineModel::setWeatherForecastManager(WeatherForecastManager *mgr)
 {
+    if (m_weatherMgr == mgr) {
+        return;
+    }
+
     m_weatherMgr = mgr;
     updateWeatherElements();
     connect(m_weatherMgr, &WeatherForecastManager::forecastUpdated, this, &TimelineModel::updateWeatherElements);
+    Q_EMIT setupChanged();
 }
 
-TripGroupManager* TimelineModel::tripGroupManager() const
+TripGroupManager *TimelineModel::tripGroupManager() const
 {
     return m_tripGroupManager;
 }
 
 void TimelineModel::setTripGroupManager(TripGroupManager *mgr)
 {
+    if (m_tripGroupManager == mgr) {
+        return;
+    }
+
     m_tripGroupManager = mgr;
     connect(mgr, &TripGroupManager::tripGroupAdded, this, &TimelineModel::tripGroupAdded);
     connect(mgr, &TripGroupManager::tripGroupChanged, this, &TimelineModel::tripGroupChanged);
     connect(mgr, &TripGroupManager::tripGroupRemoved, this, &TimelineModel::tripGroupRemoved);
-    for (const auto &group : mgr->tripGroups()) {
-        tripGroupAdded(group);
-    }
+    Q_EMIT setupChanged();
+    QMetaObject::invokeMethod(this, &TimelineModel::populate, Qt::QueuedConnection);
 }
 
 void TimelineModel::setHomeCountryIsoCode(const QString &isoCode)
 {
+    if (m_homeCountry == isoCode) {
+        return;
+    }
+
     m_homeCountry = isoCode;
     updateInformationElements();
+
+    Q_EMIT setupChanged();
 }
 
 void TimelineModel::setTransferManager(TransferManager *mgr)
 {
+    if (m_transferManager == mgr) {
+        return;
+    }
+
     m_transferManager = mgr;
     connect(mgr, &TransferManager::transferAdded, this, &TimelineModel::transferChanged);
     connect(mgr, &TransferManager::transferChanged, this, &TimelineModel::transferChanged);
     connect(mgr, &TransferManager::transferRemoved, this, &TimelineModel::transferRemoved);
-
-    // load existing transfers into the model
-    for (const auto &batchId : m_resMgr->batches()) {
-        updateTransfersForBatch(batchId);
-    }
+    Q_EMIT setupChanged();
+    QMetaObject::invokeMethod(this, &TimelineModel::populate, Qt::QueuedConnection);
 }
 
-int TimelineModel::rowCount(const QModelIndex& parent) const
+void TimelineModel::setTripGroupId(const QString &tgId)
+{
+    if (m_tripGroupId == tgId) {
+        return;
+    }
+
+    m_tripGroupId = tgId;
+    m_tripGroup = m_tripGroupManager->tripGroup(tgId);
+    Q_EMIT tripGroupIdChanged();
+
+    if (m_isPopulated) {
+        beginResetModel();
+        m_elements.clear();
+        m_isPopulated = false;
+        endResetModel();
+    }
+    QMetaObject::invokeMethod(this, &TimelineModel::populate, Qt::QueuedConnection);
+}
+
+int TimelineModel::rowCount(const QModelIndex &parent) const
 {
     if (parent.isValid() || !m_resMgr) {
         return 0;
     }
-    return m_elements.size();
+    return (int)m_elements.size();
 }
 
-QVariant TimelineModel::data(const QModelIndex& index, int role) const
+QVariant TimelineModel::data(const QModelIndex &index, int role) const
 {
     if (!index.isValid() || !m_resMgr) {
         return {};
@@ -202,66 +200,68 @@ QVariant TimelineModel::data(const QModelIndex& index, int role) const
 
     const auto &elem = m_elements.at(index.row());
     switch (role) {
-        case SectionHeaderRole:
-            // see TimelineSectionDelegateController
-            return elem.dt.date().toString(Qt::ISODate);
-        case BatchIdRole:
-            return elem.isReservation() ? elem.batchId() : QString();
-        case ElementTypeRole:
-            return elem.elementType;
-        case TodayEmptyRole:
-            if (elem.elementType == TimelineElement::TodayMarker) {
-                return isDateEmpty(m_elements.at(index.row()).dt.date());
-            }
-            return {};
-        case IsTodayRole:
-            return elem.dt.date() == today();
-        case ElementRangeRole:
-            return elem.rangeType;
-        case LocationInformationRole:
-            if (elem.elementType == TimelineElement::LocationInfo)
-                return elem.content();
-            break;
-        case WeatherForecastRole:
-            if (elem.elementType == TimelineElement::WeatherForecast)
-                return elem.content();
-            break;
-        case ReservationsRole:
-        {
-            if (!elem.isReservation()) {
-                return {};
-            }
-            const auto resIds = m_resMgr->reservationsForBatch(elem.batchId());
-            QList<QVariant> v;
-            v.reserve(resIds.size());
-            for (const auto &resId : resIds) {
-                v.push_back(m_resMgr->reservation(resId));
-            }
-            std::sort(v.begin(), v.end(), SortUtil::isBefore);
-            return QVariant::fromValue(v);
+    case SectionHeaderRole:
+        // see TimelineSectionDelegateController
+        return elem.dt.date().toString(Qt::ISODate);
+    case BatchIdRole:
+        return elem.isReservation() ? elem.batchId() : QString();
+    case ElementTypeRole:
+        return elem.elementType;
+    case TodayEmptyRole:
+        if (elem.elementType == TimelineElement::TodayMarker) {
+            return isDateEmpty(m_elements.at(index.row()).dt.date());
         }
-        case TripGroupIdRole:
-            if (elem.elementType == TimelineElement::TripGroup) {
-                return elem.content();
-            }
-            break;
-        case TripGroupRole:
-            if (elem.elementType == TimelineElement::TripGroup)
-                return QVariant::fromValue(m_tripGroupManager->tripGroup(elem.content().toString()));
-            break;
-        case TransferRole:
-            if (elem.elementType == TimelineElement::Transfer) {
-                return elem.content();
-            }
-            break;
-        case StartDateTimeRole:
-            return elem.dt;
-        case EndDateTimeRole:
-            return elem.endDateTime();
-        case IsTimeboxedRole:
-            return elem.isTimeBoxed();
-        case IsCanceledRole:
-            return elem.isCanceled();
+        return {};
+    case IsTodayRole:
+        return elem.dt.date() == today();
+    case ElementRangeRole:
+        return elem.rangeType;
+    case LocationInformationRole:
+        if (elem.elementType == TimelineElement::LocationInfo) {
+            return elem.content();
+        }
+        break;
+    case WeatherForecastRole:
+        if (elem.elementType == TimelineElement::WeatherForecast) {
+            return elem.content();
+        }
+        break;
+    case ReservationsRole: {
+        if (!elem.isReservation()) {
+            return {};
+        }
+        const auto resIds = m_resMgr->reservationsForBatch(elem.batchId());
+        QList<QVariant> v;
+        v.reserve(resIds.size());
+        for (const auto &resId : resIds) {
+            v.push_back(m_resMgr->reservation(resId));
+        }
+        std::sort(v.begin(), v.end(), SortUtil::isBefore);
+        return QVariant::fromValue(v);
+    }
+    case TripGroupIdRole:
+        if (elem.elementType == TimelineElement::TripGroup) {
+            return elem.content();
+        }
+        break;
+    case TripGroupRole:
+        if (elem.elementType == TimelineElement::TripGroup) {
+            return QVariant::fromValue(m_tripGroupManager->tripGroup(elem.content().toString()));
+        }
+        break;
+    case TransferRole:
+        if (elem.elementType == TimelineElement::Transfer) {
+            return elem.content();
+        }
+        break;
+    case StartDateTimeRole:
+        return elem.dt;
+    case EndDateTimeRole:
+        return elem.endDateTime();
+    case IsTimeboxedRole:
+        return elem.isTimeBoxed();
+    case IsCanceledRole:
+        return elem.isCanceled();
     }
     return {};
 }
@@ -286,12 +286,75 @@ QHash<int, QByteArray> TimelineModel::roleNames() const
 
 int TimelineModel::todayRow() const
 {
-    const auto it = std::find_if(m_elements.begin(), m_elements.end(), [](const auto &e) { return e.elementType == TimelineElement::TodayMarker; });
-    return std::distance(m_elements.begin(), it);
+    const auto it = std::find_if(m_elements.begin(), m_elements.end(), [](const auto &e) {
+        return e.elementType == TimelineElement::TodayMarker;
+    });
+    return it != m_elements.end() ? (int)std::distance(m_elements.begin(), it) : -1;
+}
+
+void TimelineModel::populate()
+{
+    if (m_isPopulated || !m_resMgr || !m_transferManager || !m_tripGroupManager) {
+        return;
+    }
+    m_isPopulated = true;
+
+    beginResetModel();
+    if (m_tripGroupId.isEmpty()) {
+        for (const auto &resId : m_resMgr->batches()) {
+            populateReservation(resId);
+        }
+    } else {
+        m_tripGroup = m_tripGroupManager->tripGroup(m_tripGroupId);
+        const auto elems = m_tripGroup.elements();
+        for (const auto &resId : elems) {
+            populateReservation(resId);
+        }
+    }
+    std::sort(m_elements.begin(), m_elements.end());
+    endResetModel();
+
+    // load existing transfers and trip groups into the model
+    if (m_tripGroupId.isEmpty()) {
+        for (const auto &batchId : m_resMgr->batches()) {
+            updateTransfersForBatch(batchId);
+        }
+        for (const auto &group : m_tripGroupManager->tripGroups()) {
+            tripGroupAdded(group);
+        }
+    } else {
+        const auto elems = m_tripGroup.elements();
+        for (const auto &batchId : elems) {
+            updateTransfersForBatch(batchId);
+        }
+    }
+
+    updateTodayMarker();
+    updateInformationElements();
+    Q_EMIT todayRowChanged();
+}
+
+void TimelineModel::populateReservation(const QString &resId)
+{
+    const auto res = m_resMgr->reservation(resId);
+    auto elem = TimelineElement(this, resId, res, TimelineElement::SelfContained);
+    if (!elem.isReservation()) { // a type we can't handle
+        return;
+    }
+    if (needsSplitting(res)) {
+        m_elements.emplace_back(this, resId, res, TimelineElement::RangeBegin);
+        m_elements.emplace_back(this, resId, res, TimelineElement::RangeEnd);
+    } else {
+        m_elements.push_back(std::move(elem));
+    }
 }
 
 void TimelineModel::batchAdded(const QString &resId)
 {
+    if (!m_isPopulated || (!m_tripGroupId.isEmpty() && !m_tripGroup.elements().contains(resId))) {
+        return;
+    }
+
     const auto res = m_resMgr->reservation(resId);
     if (needsSplitting(res)) {
         insertElement(TimelineElement{this, resId, res, TimelineElement::RangeBegin});
@@ -303,23 +366,21 @@ void TimelineModel::batchAdded(const QString &resId)
     updateInformationElements();
     updateTransfersForBatch(resId);
     Q_EMIT todayRowChanged();
-
-    scheduleCurrentBatchTimer();
-    Q_EMIT currentBatchChanged();
 }
 
-void TimelineModel::insertElement(TimelineElement &&elem)
+int TimelineModel::insertElement(TimelineElement &&elem)
 {
     if (elem.elementType == TimelineElement::Undefined) {
-        return;
+        return -1;
     }
 
     auto it = std::lower_bound(m_elements.begin(), m_elements.end(), elem);
-    const auto row = std::distance(m_elements.begin(), it);
+    const auto row = (int)std::distance(m_elements.begin(), it);
 
     beginInsertRows({}, row, row);
     m_elements.insert(it, std::move(elem));
     endInsertRows();
+    return row;
 }
 
 std::vector<TimelineElement>::iterator TimelineModel::insertOrUpdate(std::vector<TimelineElement>::iterator it, TimelineElement &&elem)
@@ -331,11 +392,11 @@ std::vector<TimelineElement>::iterator TimelineModel::insertOrUpdate(std::vector
     }
 
     if (it != m_elements.end() && (*it) == elem) {
-        const auto row = std::distance(m_elements.begin(), it);
+        const auto row = (int)std::distance(m_elements.begin(), it);
         (*it) = std::move(elem);
         Q_EMIT dataChanged(index(row, 0), index(row, 0));
     } else {
-        const auto row = std::distance(m_elements.begin(), it);
+        const auto row = (int)std::distance(m_elements.begin(), it);
         beginInsertRows({}, row, row);
         it = m_elements.insert(it, std::move(elem));
         endInsertRows();
@@ -345,6 +406,10 @@ std::vector<TimelineElement>::iterator TimelineModel::insertOrUpdate(std::vector
 
 void TimelineModel::batchChanged(const QString &resId)
 {
+    if (!m_isPopulated || (!m_tripGroupId.isEmpty() && !m_tripGroup.elements().contains(resId))) {
+        return;
+    }
+
     const auto res = m_resMgr->reservation(resId);
     if (needsSplitting(res)) {
         updateElement(resId, res, TimelineElement::RangeBegin);
@@ -354,20 +419,21 @@ void TimelineModel::batchChanged(const QString &resId)
     }
 
     updateInformationElements();
-
-    scheduleCurrentBatchTimer();
-    Q_EMIT currentBatchChanged();
 }
 
-void TimelineModel::batchRenamed(const QString& oldBatchId, const QString& newBatchId)
+void TimelineModel::batchRenamed(const QString &oldBatchId, const QString &newBatchId)
 {
+    if (!m_isPopulated || !m_tripGroupId.isEmpty()) {
+        return;
+    }
+
     for (auto it = m_elements.begin(); it != m_elements.end(); ++it) {
         if (!(*it).isReservation() || (*it).batchId() != oldBatchId) {
             continue;
         }
 
         (*it).setContent(newBatchId);
-        const auto idx = index(std::distance(m_elements.begin(), it), 0);
+        const auto idx = index((int)std::distance(m_elements.begin(), it), 0);
         Q_EMIT dataChanged(idx, idx);
 
         if ((*it).rangeType == TimelineElement::SelfContained || (*it).rangeType == TimelineElement::RangeEnd) {
@@ -384,7 +450,7 @@ void TimelineModel::updateElement(const QString &resId, const QVariant &res, Tim
     if (it == m_elements.end()) {
         return;
     }
-    const auto row = std::distance(m_elements.begin(), it);
+    const auto row = (int)std::distance(m_elements.begin(), it);
     const auto newDt = TimelineElement::relevantDateTime(res, rangeType);
 
     if ((*it).dt != newDt) {
@@ -400,6 +466,10 @@ void TimelineModel::updateElement(const QString &resId, const QVariant &res, Tim
 
 void TimelineModel::batchRemoved(const QString &resId)
 {
+    if (!m_isPopulated) {
+        return;
+    }
+
     const auto it = std::find_if(m_elements.begin(), m_elements.end(), [resId](const auto &e) {
         return e.isReservation() && e.batchId() == resId;
     });
@@ -407,7 +477,7 @@ void TimelineModel::batchRemoved(const QString &resId)
         return;
     }
     const auto isSplit = (*it).rangeType == TimelineElement::RangeBegin;
-    const auto row = std::distance(m_elements.begin(), it);
+    const auto row = (int)std::distance(m_elements.begin(), it);
 
     beginRemoveRows({}, row, row);
     m_elements.erase(it);
@@ -419,9 +489,6 @@ void TimelineModel::batchRemoved(const QString &resId)
     }
 
     updateInformationElements();
-
-    scheduleCurrentBatchTimer();
-    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::dayChanged()
@@ -431,43 +498,48 @@ void TimelineModel::dayChanged()
 
     m_dayUpdateTimer.setInterval((QTime::currentTime().secsTo({23, 59, 59}) + 1) * 1000);
     m_dayUpdateTimer.start();
-
-    scheduleCurrentBatchTimer();
-    Q_EMIT currentBatchChanged();
 }
 
 void TimelineModel::updateTodayMarker()
 {
+    int newRow = -1;
     auto dt = now();
     auto it = std::lower_bound(m_elements.begin(), m_elements.end(), dt);
 
-    if (it != m_elements.begin()) {
-        const auto prevIt = std::prev(it);
-        // check if the previous element is the old today marker, if so nothing to do
-        if ((*prevIt).elementType == TimelineElement::TodayMarker) {
-            (*prevIt).dt = dt;
-            return;
-        }
-        // check if the previous element is still ongoing, in that case we want to be before that
-        if ((*prevIt).dt.date() == today() && ((*prevIt).isTimeBoxed() && (*prevIt).endDateTime() > now())) {
-            it = prevIt;
-            dt = (*prevIt).dt;
+    if ((!m_tripGroup.beginDateTime().isValid() || m_tripGroup.beginDateTime().date() <= dt.date())
+        && (!m_tripGroup.endDateTime().isValid() || dt.date() <= m_tripGroup.endDateTime().date())) {
+        if (it != m_elements.begin()) {
+            const auto prevIt = std::prev(it);
+            // check if the previous element is the old today marker, if so nothing to do
+            if ((*prevIt).elementType == TimelineElement::TodayMarker) {
+                (*prevIt).dt = dt;
+                return;
+            }
+            // check if the previous element is still ongoing, in that case we want to be before that
+            if ((*prevIt).dt.date() == today() && ((*prevIt).isTimeBoxed() && (*prevIt).endDateTime() > now())) {
+                it = prevIt;
+                dt = (*prevIt).dt;
+            }
         }
+        newRow = (int)std::distance(m_elements.begin(), it);
     }
 
-    const auto newRow = std::distance(m_elements.begin(), it);
     const auto oldRow = todayRow();
     if (oldRow >= newRow) {
         return;
     }
 
-    beginInsertRows({}, newRow, newRow);
-    m_elements.insert(it, TimelineElement{this, TimelineElement::TodayMarker, dt});
-    endInsertRows();
+    if (newRow >= 0) {
+        beginInsertRows({}, newRow, newRow);
+        m_elements.insert(it, TimelineElement{this, TimelineElement::TodayMarker, dt});
+        endInsertRows();
+    }
 
-    beginRemoveRows({}, oldRow, oldRow);
-    m_elements.erase(m_elements.begin() + oldRow);
-    endRemoveRows();
+    if (oldRow >= 0) {
+        beginRemoveRows({}, oldRow, oldRow);
+        m_elements.erase(m_elements.begin() + oldRow);
+        endRemoveRows();
+    }
     Q_EMIT todayRowChanged();
 }
 
@@ -476,7 +548,7 @@ void TimelineModel::updateInformationElements()
     // the location information is shown after location changes or before stationary elements
     // when transitioning into a location that:
     // - differs in one or more properties from the home country, and the difference
-    // was introduced by this transtion
+    // was introduced by this transition
     // - differs in timezone from the previous location, and that timezone has a different
     // offset at the time of transition
 
@@ -486,7 +558,7 @@ void TimelineModel::updateInformationElements()
     auto previousCountry = homeCountry;
     for (auto it = m_elements.begin(); it != m_elements.end();) {
         if ((*it).elementType == TimelineElement::LocationInfo) { // this is one we didn't generate, otherwise it would be beyond that
-            const auto row = std::distance(m_elements.begin(), it);
+            const auto row = (int)std::distance(m_elements.begin(), it);
             beginRemoveRows({}, row, row);
             it = m_elements.erase(it);
             endRemoveRows();
@@ -532,22 +604,32 @@ void TimelineModel::updateInformationElements()
         qDebug() << "looking for DST transitions";
         constexpr const auto DST_DAYS_BEFORE = 30;
         constexpr const auto DST_DAYS_AFTER = 90;
+        auto searchWindowBegin = now().addDays(-DST_DAYS_BEFORE);
+        if (m_tripGroup.beginDateTime().isValid()) {
+            searchWindowBegin = std::max(searchWindowBegin, m_tripGroup.beginDateTime());
+        }
+        auto searchWindowEnd = now().addDays(DST_DAYS_AFTER);
+        if (m_tripGroup.endDateTime().isValid()) {
+            searchWindowEnd = std::min(searchWindowEnd, m_tripGroup.beginDateTime());
+        }
+
         for (auto it = m_elements.begin(); it != m_elements.end(); ++it) {
             if ((*it).isInformational() || (*it).isCanceled()) {
                 continue;
             }
             const auto tz = timeZone((*it).endDateTime());
-            auto startDt = std::max((*it).dt, now().addDays(-DST_DAYS_BEFORE));
-            auto endDt = now().addDays(DST_DAYS_AFTER);
+            auto startDt = std::max((*it).dt, searchWindowBegin);
+            auto endDt = searchWindowEnd;
             for (auto nextIt = it; std::next(nextIt) != m_elements.end();) {
                 ++nextIt;
-                if ((*nextIt).isInformational() || (*nextIt).isCanceled()) {
+                if ((*nextIt).isInformational() || (*nextIt).isCanceled()
+                    || ((*nextIt).elementType == TimelineElement::Transfer && (*nextIt).content().value<Transfer>().state() != Transfer::Selected)) {
                     continue;
                 }
                 endDt = std::min(endDt, (*nextIt).dt);
                 break;
             }
-            if (startDt > now().addDays(DST_DAYS_AFTER) || endDt < now().addDays(-DST_DAYS_BEFORE) || !tz.isValid() || !tz.hasTransitions() || !tz.hasDaylightTime()) {
+            if (startDt > searchWindowEnd || endDt < searchWindowBegin || !tz.isValid() || !tz.hasTransitions() || !tz.hasDaylightTime()) {
                 continue;
             }
             while (startDt < endDt) {
@@ -575,20 +657,26 @@ void TimelineModel::updateWeatherElements()
     }
 
     qDebug() << "recomputing weather elements";
-    GeoCoordinates geo;
+    GeoCoordinates geo = LocationUtil::geo(m_initialLocation);
     QString label;
 
     auto date = now();
+    if (m_tripGroup.beginDateTime().isValid()) {
+        date = std::max(date, m_tripGroup.beginDateTime());
+    }
     // round to next full hour
     date.setTime(QTime(date.time().hour(), 0));
     date = date.addSecs(60 * 60);
-    const auto maxForecastTime = m_weatherMgr->maximumForecastTime(date.date());
+    auto maxForecastTime = m_weatherMgr->maximumForecastTime(date.date());
+    if (m_tripGroup.endDateTime().isValid()) {
+        maxForecastTime = std::min(maxForecastTime, m_tripGroup.endDateTime());
+    }
 
     // look through the past, clean up weather elements there and figure out where we are
     auto it = m_elements.begin();
     for (; it != m_elements.end() && (*it).dt < now();) {
         if ((*it).elementType == TimelineElement::WeatherForecast) {
-            const auto row = std::distance(m_elements.begin(), it);
+            const auto row = (int)std::distance(m_elements.begin(), it);
             beginRemoveRows({}, row, row);
             it = m_elements.erase(it);
             endRemoveRows();
@@ -614,12 +702,11 @@ void TimelineModel::updateWeatherElements()
         ++it;
     }
 
-    while(it != m_elements.end() && date < maxForecastTime) {
-
+    while (it != m_elements.end() && date < maxForecastTime) {
         if ((*it).dt < date || (*it).elementType == TimelineElement::TodayMarker) {
             // clean up outdated weather elements (happens when merging previously split ranges)
             if ((*it).elementType == TimelineElement::WeatherForecast) {
-                const auto row = std::distance(m_elements.begin(), it);
+                const auto row = (int)std::distance(m_elements.begin(), it);
                 beginRemoveRows({}, row, row);
                 it = m_elements.erase(it);
                 endRemoveRows();
@@ -676,7 +763,7 @@ void TimelineModel::updateWeatherElements()
         }
         // we have no forecast data, but a matching weather element: remove
         else if ((*it).elementType == TimelineElement::WeatherForecast && (*it).dt == date) {
-            const auto row = std::distance(m_elements.begin(), it);
+            const auto row = (int)std::distance(m_elements.begin(), it);
             beginRemoveRows({}, row, row);
             it = m_elements.erase(it);
             endRemoveRows();
@@ -696,7 +783,7 @@ void TimelineModel::updateWeatherElements()
         m_weatherMgr->monitorLocation(geo.latitude(), geo.longitude());
         const auto fc = m_weatherMgr->forecast(geo.latitude(), geo.longitude(), date, endTime);
         if (fc.isValid()) {
-            const auto row = std::distance(m_elements.begin(), it);
+            const auto row = (int)std::distance(m_elements.begin(), it);
             beginInsertRows({}, row, row);
             it = m_elements.insert(it, TimelineElement{this, TimelineElement::WeatherForecast, date, QVariant::fromValue(WeatherInformation{fc, label})});
             ++it;
@@ -708,7 +795,7 @@ void TimelineModel::updateWeatherElements()
     qDebug() << "weather recomputation done";
 }
 
-void TimelineModel::updateTransfersForBatch(const QString& batchId)
+void TimelineModel::updateTransfersForBatch(const QString &batchId)
 {
     if (!m_transferManager) {
         return;
@@ -747,48 +834,92 @@ void TimelineModel::setCurrentDateTime(const QDateTime &dt)
     if (dayDiffers && !m_elements.empty()) {
         dayChanged();
     }
-
-    scheduleCurrentBatchTimer();
 }
 
-void TimelineModel::tripGroupAdded(const QString& groupId)
+void TimelineModel::tripGroupAdded(const QString &groupId)
 {
+    if (!m_isPopulated || !m_tripGroupId.isEmpty()) {
+        return;
+    }
+
     const auto g = m_tripGroupManager->tripGroup(groupId);
+    if (g.elements().size() < 2) {
+        return;
+    }
 
     TimelineElement beginElem{this, TimelineElement::TripGroup, g.beginDateTime(), groupId};
     beginElem.rangeType = TimelineElement::RangeBegin;
-    insertElement(std::move(beginElem));
+    const auto startRow = insertElement(std::move(beginElem));
 
     TimelineElement endElem{this, TimelineElement::TripGroup, g.endDateTime(), groupId};
     endElem.rangeType = TimelineElement::RangeEnd;
-    insertElement(std::move(endElem));
-}
+    const auto endRow = insertElement(std::move(endElem));
 
-void TimelineModel::tripGroupChanged(const QString& groupId)
-{
-    // ### this can be done better probably
-    tripGroupRemoved(groupId);
-    tripGroupAdded(groupId);
-}
-
-void TimelineModel::tripGroupRemoved(const QString& groupId)
-{
-    for (auto it = m_elements.begin(); it != m_elements.end();) {
-        if ((*it).elementType != TimelineElement::TripGroup || (*it).content().toString() != groupId) {
-            ++it;
-            continue;
-        }
-
-        const auto row = std::distance(m_elements.begin(), it);
-        beginRemoveRows({}, row, row);
-        it = m_elements.erase(it);
-        endRemoveRows();
+    // elements in that group now have a trip group id
+    if (endRow - startRow > 1) {
+        const auto beginIdx = index(startRow + 1, 0);
+        const auto endIdx = index(endRow - 1, 0);
+        Q_EMIT dataChanged(beginIdx, endIdx, {TimelineModel::TripGroupIdRole});
     }
 }
 
-void TimelineModel::transferChanged(const Transfer& transfer)
+void TimelineModel::tripGroupChanged(const QString &groupId)
 {
-    if (transfer.state() == Transfer::UndefinedState) {
+    if (!m_isPopulated) {
+        return;
+    }
+
+    if (m_tripGroupId.isEmpty()) {
+        // ### this can be done better probably
+        tripGroupRemoved(groupId);
+        tripGroupAdded(groupId);
+    } else if (m_tripGroupId == groupId) {
+        const auto oldElems = m_tripGroup.elements();
+        m_tripGroup = m_tripGroupManager->tripGroup(groupId);
+        const auto newElems = m_tripGroup.elements();
+
+        for (const auto &oldBatchId : oldElems) {
+            if (!newElems.contains(oldBatchId)) {
+                batchRemoved(oldBatchId);
+            }
+        }
+        for (const auto &newBatchId : newElems) {
+            if (!oldElems.contains(newBatchId)) {
+                batchAdded(newBatchId);
+            }
+        }
+    }
+}
+
+void TimelineModel::tripGroupRemoved(const QString &groupId)
+{
+    if (!m_isPopulated) {
+        return;
+    }
+
+    if (m_tripGroupId.isEmpty()) {
+        for (auto it = m_elements.begin(); it != m_elements.end();) {
+            if ((*it).elementType != TimelineElement::TripGroup || (*it).content().toString() != groupId) {
+                ++it;
+                continue;
+            }
+
+            const auto row = (int)std::distance(m_elements.begin(), it);
+            beginRemoveRows({}, row, row);
+            it = m_elements.erase(it);
+            endRemoveRows();
+        }
+    } else if (m_tripGroupId == groupId) {
+        beginResetModel();
+        m_elements.clear();
+        m_isPopulated = false;
+        endResetModel();
+    }
+}
+
+void TimelineModel::transferChanged(const Transfer &transfer)
+{
+    if (!m_isPopulated || transfer.state() == Transfer::UndefinedState) {
         return;
     }
     if (transfer.state() == Transfer::Discarded) {
@@ -820,15 +951,18 @@ void TimelineModel::transferChanged(const Transfer& transfer)
 
 void TimelineModel::transferRemoved(const QString &resId, Transfer::Alignment alignment)
 {
+    if (!m_isPopulated) {
+        return;
+    }
+
     auto it = std::find_if(m_elements.begin(), m_elements.end(), [resId, alignment](const auto &e) {
-        return e.elementType == TimelineElement::Transfer && e.batchId() == resId
-            && e.content().template value<Transfer>().alignment() == alignment;
+        return e.elementType == TimelineElement::Transfer && e.batchId() == resId && e.content().template value<Transfer>().alignment() == alignment;
     });
     if (it == m_elements.end()) {
         return;
     }
 
-    const auto row = std::distance(m_elements.begin(), it);
+    const auto row = (int)std::distance(m_elements.begin(), it);
     beginRemoveRows({}, row, row);
     m_elements.erase(it);
     endRemoveRows();
@@ -836,167 +970,6 @@ void TimelineModel::transferRemoved(const QString &resId, Transfer::Alignment al
     Q_EMIT todayRowChanged();
 }
 
-[[nodiscard]] static bool isSelectableElement(const TimelineElement &elem)
-{
-    return elem.isReservation() && elem.rangeType == TimelineElement::SelfContained;
-}
-
-// same as SortUtil::endDateTime but with a lower bound estimate
-// when the end time isn't available
-[[nodiscard]] static QDateTime estimatedEndTime(const QVariant &res)
-{
-    if (SortUtil::hasEndTime(res)) {
-        return SortUtil::endDateTime(res);
-    }
-    if (JsonLd::isA<FlightReservation>(res)) {
-        const auto flight = res.value<FlightReservation>().reservationFor().value<Flight>();
-        const auto dist = LocationUtil::distance(flight.departureAirport().geo(), flight.arrivalAirport().geo());
-        if (std::isnan(dist) || !flight.departureTime().isValid()) {
-            return {};
-        }
-        auto dt = flight.departureTime();
-        return dt.addSecs((qint64)(dist * 250.0 / 3.6)); // see flightutil.cpp in kitinerary
-    }
-
-    return {};
-}
-
-QString TimelineModel::currentBatchId() const
-{
-    if (m_elements.empty()) {
-        return {};
-    }
-
-    // find the next reservation
-    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), now());
-    for (; it != m_elements.end() && !isSelectableElement(*it); ++it) {}
-
-    QString nextResId;
-    QDateTime nextStartTime;
-    if (it != m_elements.end() && now().secsTo((*it).dt) < Constants::CurrentBatchLeadingMargin.count()) {
-        nextResId = (*it).batchId();
-        nextStartTime = (*it).dt;
-    }
-
-    // find the previous or current reservation
-    if (it != m_elements.begin()) {
-        --it;
-        for (; it != m_elements.begin() && (it == m_elements.end() || !isSelectableElement(*it)); --it) {}
-    }
-
-    auto resId = (*it).batchId();
-    const auto res = m_resMgr->reservation(resId);
-    auto endTime = estimatedEndTime(res);
-    if (endTime.secsTo(now()) > Constants::CurrentBatchTrailingMargin.count()) {
-        endTime = {};
-    }
-
-    // only one side found
-    if (!endTime.isValid()) {
-        return nextResId;
-    }
-    if (!nextStartTime.isValid()) {
-        return resId;
-    }
-
-    // (*it) is still active
-    if (endTime >= now()) {
-        return resId;
-    }
-
-    // take the one that is closer
-    return endTime.secsTo(now()) < now().secsTo(nextStartTime) ? resId : nextResId;
-}
-
-void TimelineModel::scheduleCurrentBatchTimer()
-{
-    if (m_elements.empty()) {
-        return;
-    }
-
-    // we need the smallest valid time > now() of any of the following:
-    // - end time of the current element + margin
-    // - start time - margin of the next res
-    // - end time of current + start time of next - endtime of current / 2
-    // - end time of the next element (in case we are in the leading margin already)
-
-    QDateTime triggerTime;
-    const auto updateTriggerTime = [&triggerTime, this](const QDateTime &dt) {
-        if (!dt.isValid() || dt <= now()) {
-            return;
-        }
-        if (!triggerTime.isValid()) {
-            triggerTime = dt;
-        } else {
-            triggerTime = std::min(triggerTime, dt);
-        }
-    };
-
-    // find the next reservation
-    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), now());
-    for (; it != m_elements.end() && !isSelectableElement(*it); ++it) {}
-
-    QDateTime nextStartTime;
-    if (it != m_elements.end()) {
-        nextStartTime = (*it).dt;
-        updateTriggerTime(nextStartTime.addSecs(-Constants::CurrentBatchLeadingMargin.count()));
-        updateTriggerTime(SortUtil::endDateTime(m_resMgr->reservation((*it).batchId())));
-    }
-
-    // find the previous or current reservation
-    if (it != m_elements.begin()) {
-        --it;
-        for (; it != m_elements.begin() && (it == m_elements.end() || !isSelectableElement(*it)); --it) {}
-    }
-
-    const auto res = m_resMgr->reservation((*it).batchId());
-    auto endTime = SortUtil::endDateTime(res);
-    updateTriggerTime(endTime.addSecs(Constants::CurrentBatchTrailingMargin.count()));
-
-    if (nextStartTime.isValid() && endTime.isValid()) {
-        updateTriggerTime(endTime.addSecs(endTime.secsTo(nextStartTime) / 2));
-    }
-
-    // QTimer only has 31bit for its msec interval, so don't schedule beyond a day
-    // for longer distances we re-run this in the midnight timer above
-    if (triggerTime.isValid() && triggerTime.date() == today()) {
-        m_currentBatchTimer.setInterval(std::chrono::seconds(std::max<qint64>(60, now().secsTo(triggerTime))));
-        m_currentBatchTimer.start();
-    }
-}
-
-QVariant TimelineModel::locationAtTime(const QDateTime& dt) const
-{
-    auto it = std::lower_bound(m_elements.begin(), m_elements.end(), dt);
-    if (it == m_elements.begin()) {
-        return {};
-    }
-
-    for (--it ;; --it) {
-        // this is a still ongoing non-location change
-        if (it != m_elements.end() && !(*it).isLocationChange() && (*it).endDateTime().isValid() && (*it).endDateTime() > dt) {
-            if ((*it).isReservation()) {
-                auto loc = LocationUtil::location(m_resMgr->reservation((*it).batchId()));
-                if (LocationUtil::geo(loc).isValid() || !LocationUtil::address(loc).addressCountry().isEmpty()) {
-                    return loc;
-                }
-            }
-        }
-
-        if ((*it).isReservation() && (*it).isLocationChange()) {
-            // TODO make this work for transfers too
-            const auto res = m_resMgr->reservation((*it).batchId());
-            return LocationUtil::arrivalLocation(res);
-        }
-
-        if (it == m_elements.begin()) {
-            break;
-        }
-    }
-
-    return {};
-}
-
 bool TimelineModel::isDateEmpty(const QDate &date) const
 {
     auto it = std::lower_bound(m_elements.begin(), m_elements.end(), date, [](const auto &lhs, auto rhs) {
